Loosen the requirements for atomic and legacy commit so that, in cases
where solid fill planes is enabled (and FB_ID is NULL), the commit can
still go through.

In addition, add framebuffer NULL checks in other areas to account for
FB being NULL when solid fill is enabled.

Signed-off-by: Jessica Zhang <quic_jessz...@quicinc.com>
---
 drivers/gpu/drm/drm_atomic.c        | 68 ++++++++++++++++-------------
 drivers/gpu/drm/drm_atomic_helper.c | 34 +++++++++------
 drivers/gpu/drm/drm_plane.c         |  8 ++--
 include/drm/drm_atomic_helper.h     |  5 ++-
 4 files changed, 64 insertions(+), 51 deletions(-)

diff --git a/drivers/gpu/drm/drm_atomic_helper.c 
b/drivers/gpu/drm/drm_atomic_helper.c
index 8bf41aa24068..5a5ffa06b8bd 100644
--- a/drivers/gpu/drm/drm_atomic_helper.c
+++ b/drivers/gpu/drm/drm_atomic_helper.c
@@ -818,7 +818,7 @@ int drm_atomic_helper_check_plane_state(struct 
drm_plane_state *plane_state,
        *src = drm_plane_state_src(plane_state);
        *dst = drm_plane_state_dest(plane_state);
 
-       if (!fb) {
+       if (!fb && !plane_state->color_fill) {
                plane_state->visible = false;
                return 0;
        }
@@ -835,25 +835,31 @@ int drm_atomic_helper_check_plane_state(struct 
drm_plane_state *plane_state,
                return -EINVAL;
        }
 
-       drm_rect_rotate(src, fb->width << 16, fb->height << 16, rotation);
+       if (fb) {
+               drm_rect_rotate(src, fb->width << 16, fb->height << 16, 
rotation);
 
-       /* Check scaling */
-       hscale = drm_rect_calc_hscale(src, dst, min_scale, max_scale);
-       vscale = drm_rect_calc_vscale(src, dst, min_scale, max_scale);
-       if (hscale < 0 || vscale < 0) {
-               drm_dbg_kms(plane_state->plane->dev,
-                           "Invalid scaling of plane\n");
-               drm_rect_debug_print("src: ", &plane_state->src, true);
-               drm_rect_debug_print("dst: ", &plane_state->dst, false);
-               return -ERANGE;
+               /* Check scaling */
+               hscale = drm_rect_calc_hscale(src, dst, min_scale, max_scale);
+               vscale = drm_rect_calc_vscale(src, dst, min_scale, max_scale);
+
+               if (hscale < 0 || vscale < 0) {
+                       drm_dbg_kms(plane_state->plane->dev,
+                               "Invalid scaling of plane\n");
+                       drm_rect_debug_print("src: ", &plane_state->src, true);
+                       drm_rect_debug_print("dst: ", &plane_state->dst, false);
+                       return -ERANGE;
+               }
        }
 
        if (crtc_state->enable)
                drm_mode_get_hv_timing(&crtc_state->mode, &clip.x2, &clip.y2);
 
-       plane_state->visible = drm_rect_clip_scaled(src, dst, &clip);
-
-       drm_rect_rotate_inv(src, fb->width << 16, fb->height << 16, rotation);
+       if (fb) {
+               plane_state->visible = drm_rect_clip_scaled(src, dst, &clip);
+               drm_rect_rotate_inv(src, fb->width << 16, fb->height << 16, 
rotation);
+       } else if (plane_state->color_fill) {
+               plane_state->visible = true;
+       }
 
        if (!plane_state->visible)
                /*
diff --git a/include/drm/drm_atomic_helper.h b/include/drm/drm_atomic_helper.h
index 2a0b17842402..aa7576f0879d 100644
--- a/include/drm/drm_atomic_helper.h
+++ b/include/drm/drm_atomic_helper.h
@@ -219,8 +219,9 @@ drm_atomic_plane_disabling(struct drm_plane_state 
*old_plane_state,
         * Anything else should be considered a bug in the atomic core, so we
         * gently warn about it.
         */
-       WARN_ON((new_plane_state->crtc == NULL && new_plane_state->fb != NULL) 
||
-               (new_plane_state->crtc != NULL && new_plane_state->fb == NULL));
+       WARN_ON(!new_plane_state->color_fill &&
+                       ((new_plane_state->crtc == NULL && new_plane_state->fb 
!= NULL) ||
+                       (new_plane_state->crtc != NULL && new_plane_state->fb 
== NULL)));
 
        return old_plane_state->crtc && !new_plane_state->crtc;
 }
diff --git a/drivers/gpu/drm/drm_atomic.c b/drivers/gpu/drm/drm_atomic.c
index f197f59f6d99..b576ed221431 100644
--- a/drivers/gpu/drm/drm_atomic.c
+++ b/drivers/gpu/drm/drm_atomic.c
@@ -601,8 +601,10 @@ static int drm_atomic_plane_check(const struct 
drm_plane_state *old_plane_state,
        uint32_t num_clips;
        int ret;
 
-       /* either *both* CRTC and FB must be set, or neither */
-       if (crtc && !fb) {
+       /* When color_fill is disabled,
+        * either *both* CRTC and FB must be set, or neither
+        */
+       if (crtc && !fb && !new_plane_state->color_fill) {
                drm_dbg_atomic(plane->dev, "[PLANE:%d:%s] CRTC set but no FB\n",
                               plane->base.id, plane->name);
                return -EINVAL;
@@ -626,14 +628,16 @@ static int drm_atomic_plane_check(const struct 
drm_plane_state *old_plane_state,
        }
 
        /* Check whether this plane supports the fb pixel format. */
-       ret = drm_plane_check_pixel_format(plane, fb->format->format,
-                                          fb->modifier);
-       if (ret) {
-               drm_dbg_atomic(plane->dev,
-                              "[PLANE:%d:%s] invalid pixel format %p4cc, 
modifier 0x%llx\n",
-                              plane->base.id, plane->name,
-                              &fb->format->format, fb->modifier);
-               return ret;
+       if (fb) {
+               ret = drm_plane_check_pixel_format(plane, fb->format->format,
+                                                  fb->modifier);
+
+               if (ret)
+                       drm_dbg_atomic(plane->dev,
+                                      "[PLANE:%d:%s] invalid pixel format 
%p4cc, modifier 0x%llx\n",
+                                      plane->base.id, plane->name,
+                                      &fb->format->format, fb->modifier);
+                       return ret;
        }
 
        /* Give drivers some help against integer overflows */
@@ -649,28 +653,30 @@ static int drm_atomic_plane_check(const struct 
drm_plane_state *old_plane_state,
                return -ERANGE;
        }
 
-       fb_width = fb->width << 16;
-       fb_height = fb->height << 16;
+       if (fb) {
+               fb_width = fb->width << 16;
+               fb_height = fb->height << 16;
 
-       /* Make sure source coordinates are inside the fb. */
-       if (new_plane_state->src_w > fb_width ||
-           new_plane_state->src_x > fb_width - new_plane_state->src_w ||
-           new_plane_state->src_h > fb_height ||
-           new_plane_state->src_y > fb_height - new_plane_state->src_h) {
-               drm_dbg_atomic(plane->dev,
-                              "[PLANE:%d:%s] invalid source coordinates "
-                              "%u.%06ux%u.%06u+%u.%06u+%u.%06u (fb %ux%u)\n",
-                              plane->base.id, plane->name,
-                              new_plane_state->src_w >> 16,
-                              ((new_plane_state->src_w & 0xffff) * 15625) >> 
10,
-                              new_plane_state->src_h >> 16,
-                              ((new_plane_state->src_h & 0xffff) * 15625) >> 
10,
-                              new_plane_state->src_x >> 16,
-                              ((new_plane_state->src_x & 0xffff) * 15625) >> 
10,
-                              new_plane_state->src_y >> 16,
-                              ((new_plane_state->src_y & 0xffff) * 15625) >> 
10,
-                              fb->width, fb->height);
-               return -ENOSPC;
+               /* Make sure source coordinates are inside the fb. */
+               if (new_plane_state->src_w > fb_width ||
+                   new_plane_state->src_x > fb_width - new_plane_state->src_w 
||
+                   new_plane_state->src_h > fb_height ||
+                   new_plane_state->src_y > fb_height - 
new_plane_state->src_h) {
+                       drm_dbg_atomic(plane->dev,
+                                      "[PLANE:%d:%s] invalid source 
coordinates "
+                                      "%u.%06ux%u.%06u+%u.%06u+%u.%06u (fb 
%ux%u)\n",
+                                      plane->base.id, plane->name,
+                                      new_plane_state->src_w >> 16,
+                                      ((new_plane_state->src_w & 0xffff) * 
15625) >> 10,
+                                      new_plane_state->src_h >> 16,
+                                      ((new_plane_state->src_h & 0xffff) * 
15625) >> 10,
+                                      new_plane_state->src_x >> 16,
+                                      ((new_plane_state->src_x & 0xffff) * 
15625) >> 10,
+                                      new_plane_state->src_y >> 16,
+                                      ((new_plane_state->src_y & 0xffff) * 
15625) >> 10,
+                                      fb->width, fb->height);
+                       return -ENOSPC;
+               }
        }
 
        clips = __drm_plane_get_damage_clips(new_plane_state);
diff --git a/drivers/gpu/drm/drm_plane.c b/drivers/gpu/drm/drm_plane.c
index 726f2f163c26..223b5ed93d3a 100644
--- a/drivers/gpu/drm/drm_plane.c
+++ b/drivers/gpu/drm/drm_plane.c
@@ -850,8 +850,8 @@ static int __setplane_internal(struct drm_plane *plane,
 
        WARN_ON(drm_drv_uses_atomic_modeset(plane->dev));
 
-       /* No fb means shut it down */
-       if (!fb) {
+       /* No fb and no color fill means shut it down */
+       if (!fb && (plane->state && !plane->state->color_fill)) {
                plane->old_fb = plane->fb;
                ret = plane->funcs->disable_plane(plane, ctx);
                if (!ret) {
@@ -902,8 +902,8 @@ static int __setplane_atomic(struct drm_plane *plane,
 
        WARN_ON(!drm_drv_uses_atomic_modeset(plane->dev));
 
-       /* No fb means shut it down */
-       if (!fb)
+       /* No fb and no color fill means shut it down */
+       if (!fb && (plane->state && !plane->state->color_fill))
                return plane->funcs->disable_plane(plane, ctx);
 
        /*
-- 
2.38.0

Reply via email to